(function( require ){
  
  require.config( function(){
    var href = ( window.top != window ) ? document.referrer : window.location.href;
    var cdn  = 'https://www-tc.pbskids.org';
    
    if (href.match(/^https?:\/\/((?!www(\-tc)?\.).+)pbskids\.org/)){
      // Override if on any subdomain other than 'www-tc'
      if (href.match(/soup\.pbskids\.org/))
        cdn = 'https://soup-tc.pbskids.org';
      
      else if (href.match(/ernie\.pbskids\.org\//))
        cdn = 'https://ernie-tc.pbskids.org';

      else
        cdn = 'https://' + window.location.host;
    }

    require.cdn = cdn;

    var shell_root  = cdn + (href.match(/\/~.*\/site\/?/) ? href.substring(0,href.indexOf("site") + 4) + "/" : "/shell/");
    var carson_root = cdn + (function( m ){ return ( m ? m[0] : "/shell/" ); }( href.match(/(\/~([a-zA-Z0-9]+)\/)|\/cms/) ));
    
    // DEFINE PATHS TO TORTUGA RESOURCES:
    // Only debug on subdomains of pbskids.org, e.g. stage.pbskids.org,
    // except NOT on help.pbskids.org, www-tc, cms-tc, nor cms.pbskids.org.
    var _STAGING_SUBDOMAINS = /(^((?!(www|help|cms)(\-tc)?\.).+)pbskids\.org$|tortuga\-docker\.localhost)/;
    var _TORTUGA_DEBUG = _STAGING_SUBDOMAINS.test(window.location.hostname);

    var tortuga_root  = 'https://' + (_TORTUGA_DEBUG ? 'cms-dev-tc' : 'cms-tc') + '.pbskids.org/';
    var headband_root = tortuga_root + "headband/resources/js/";
    var sponsor_root  = tortuga_root + "sponsorship/resources/js/";

    // window._headband_js_root is used by the tortuga-localization lib.
    window._headband_js_root = headband_root;

    return{
      baseUrl: cdn + '/js/',
      urlArgs: "version=1.72",
      shim: {
        'sound': {
          exports: 'createjs'
        },
        'Handlebars': {
          exports: 'Handlebars'
        },
        'faye': {
          exports: 'Faye'
        }
      },
      paths: {
        //jQuery and jQuery Plugins
        'jquery'          : 'loader/lib/jquery/jquery-1.10.2',
        'jquery-easing'   : shell_root + 'js/lib/jquery/plugins/jquery-easing-1.3',
        'jquery-touch'    : shell_root + 'js/lib/jquery/jquery.mobile-1.3.1-touch-swipe-only.min',
        'jquery-mobile'   : shell_root + 'js/lib/jquery/jquery.mobile.custom',
        'jquery-bxslider' : shell_root + 'js/lib/jquery/plugins/jquery.bxSlider.min',

        //PBS KIDS HEADBAND MODULES
        'headband'            : headband_root + 'producer-headband',
        'localization'        : headband_root + 'localization',

        //PBS KIDS MESSAGING SYSTEM
        'uuid'              : 'lib/PBS.KIDS.uuid',
        'jquery-noconflict' : headband_root + 'jquery-noconflict',
        'messages'          : headband_root + 'messages',
        'pubsub'            : headband_root + 'PBS.KIDS.pubsub',

        //Progress Tracker
        'utils'               : '/includes/progresstracker/js/utils',
        'queueingLibrary2'    : '/includes/progresstracker/js/queueingLibrary2',
        'game-tracker'        : '/includes/progresstracker/js/game-tracker',
        'identity-0.2'        : '/includes/progresstracker/js/identity-0.2',
        'simple-storage'      : cdn + '/js/lib/PBS.KIDS.simple-storage',

        //Page Views
        'shell' : shell_root + 'js/shell',

        //Other Plugins and Libs
        'images-loaded'   : shell_root + 'js/lib/imagesloaded',
        'sound'           : shell_root + 'js/lib/Sound',
        'howler'          : shell_root + 'audio/howler',
        'swf-object'      : shell_root + 'js/lib/swfobject-2.2.min',

        'kinetic'         : shell_root  + 'js/lib/kinetic-v5.0.1.min',
        'the-wheel'       : shell_root  + 'js/the-wheel/the-wheel',
        'carson'          : carson_root + 'carson/carson',

        'bridge-overlay'  : sponsor_root + 'bridge',
        'bridge-urls'     : sponsor_root + 'bridge.urls',

        'jquery-video'    : 'https://www-tc.pbskids.org/pbsk/video/lib/jquery.pbskidsvideo.min-latest',
        'progress-tracker': 'progress-tracker url has not yet been defined'
      }
    };
  }());

  //Loaders
  require( ["messages", "headband"] );

}( PBS.KIDS.require ));
